#include "opencv2/imgcodecs.hpp"
#include "opencv2/highgui.hpp"
#include "opencv2/stitching.hpp"

#include <iostream>

using namespace std;
using namespace  cv;

Stitcher::Mode mode = Stitcher::PANORAMA; // Panorama Mode for photos
// Stitcher::Mode mode = Stitcher::SCANS; // SCAN Mode for Text scans

int main(int argc, char **argv)
{
  string dirName = "scene";
  vector<Mat> images;
  vector<String> files;
  String destination = dirName + "_result.png";

  cout << "USAGE : ./panorama <path to folder containing images>";

  if( argc > 1 )
    dirName = argv[1];

  glob(dirName, files);
  std::sort(files.begin(),files.end());

  for (size_t i = 0; i < files.size(); ++i)
  {
      cout << files[i] << endl;
      Mat img = imread(files[i]); // load the image
      if (img.empty())            // invalid image, skip it.
      {
          cout << files[i] << " is invalid!" << endl;
          continue;
      }

      // Create a list of images
      images.push_back(img);
  }

  Mat panorama;
  // create stitcher object using stitcher class and mode. mode can be changed from panorama to scans.
  // check modes: https://docs.opencv.org/3.4/d2/d8d/classcv_1_1Stitcher.html#a114713924ec05a0309f4df7e918c0324
  Ptr<Stitcher> stitcher  = Stitcher::create(mode);
  Stitcher::Status status = stitcher->stitch(images, panorama);
  
  // check return error code Error Code 1 Means Stitching requires more images
  // error codes: https://docs.opencv.org/3.4/d2/d8d/classcv_1_1Stitcher.html#a507409ce9435dd89857469d12ec06b45
  if(status == Stitcher::OK) 
  {
    // write the panorama image to destination path
    imwrite(destination, panorama);
    cout << "Output image written to " << destination << endl;
    return 0;
  }

  else
  {
    cout << "Error in Stitching, error code = " << int(status) << endl;
    return -1;
  }
}
